"""
Unit and regression test for the alphaPredict package.
"""

# Import package, test suite, and other packages as needed
import alphaPredict
import pytest
import sys

def test_alphaPredict_imported():
    """Sample test, will always pass so long as import statement worked."""
    assert "alphaPredict" in sys.modules
